
from __future__ import print_function, division

import torch
import torch.nn as nn
import torch.optim as optim
from torch.optim.nadam import WGD_Nadam
from torch.optim import lr_scheduler
import sys, os, signal
import numpy as np
import torchvision
from torchvision import datasets, models, transforms
#import darknet
import time, copy, utils
from time import process_time, localtime, strftime
import vars
from train import train_model, test_model, visualize_model, test_model_mio_tcd, test_model_metric
import matplotlib.pyplot as plt
from torchsummary import summary
import mymodels
from PIL import Image
import PIL.ImageOps   
from torch.utils.data import DataLoader, Dataset

#import optimizers with GC
# from myoptimizers.SGD import mySGD
# from myoptimizers.Adam import myAdam,myAdamW
# from myoptimizers.RAdam import myRAdam
# from myoptimizers.Lookahead import Lookahead
# from myoptimizers.Ranger import Ranger
#import optimizers with GC

from algorithm.SGD import WGD_SGD,SGDW,SGDW_GCC
from algorithm.Adam import WGD_Adam,AdamW,AdamW_GCC
from algorithm.Adagrad import Adagrad_GCC
from algorithm.RAdam import RAdam, WGD_RAdam 
print(torch.__version__)

class MIO_Dataset(Dataset):
    def __init__(self, imgs, transform=None):
        """ Constructor
        Inputs:
            imgs: a list of images and their corresponding labels
            transform: required transformations
        """
        self.imgs = imgs
        self.transform = transform
        if transform is None:
            self.transform = transforms.ToTensor()
        
    def __getitem__(self, index):
        # select the image with given index from dataset
        img_path, label = self.imgs[index] 
        
        img_name = os.path.basename(img_path)[:-4]                

        # read image
        img = Image.open(img_path)
        img = img.convert('RGB')
        
        # perform given transformations (if any)
        if self.transform is not None:
            img = self.transform(img)

        return img, img_name
    
    def __len__(self):
        return len(self.imgs)


# برای تقسیم داده ها به آموزش و تست به صورت خودکار
# این تابع درصدی از داده های آموزش را ضمن حفظ اسامی پوشه ها، به پوشه تست منتقل می کند 
if(not os.path.exists(vars.val_dir)):
    q = input("create val data? (0: no, 1: yes)  ")
    if(q == '1'):
        utils.create_validation_data(vars.train_dir, vars.val_dir, vars.val_split_ratio, 'jpg')

#utils.normalize_images("E:\\Dataset\\MIO-TCD-Classification\\train", "E:\\Dataset\\MIO-TCD-Classification\\train_256")

def handler(signum, frame):
	print('Signal handler called with signal', signum)
	print('Training will finish after this epoch')
	vars.stop_training = True
	#raise OSError("Couldn't open vars.device!")

signal.signal(signal.SIGINT, handler) # only in python version >= 3.2

print("Start Time: ", strftime("%Y-%m-%d %H:%M:%S", localtime()))
print("Active Mode: " + vars.mode)
plt.ion()   # interactive mode
######################################################################
# Load Data
# Data augmentation and normalization for training
# Just normalization for validation

data_transforms = {
    'train': transforms.Compose([
        transforms.Resize([vars.input_size, vars.input_size]),
        # transforms.ColorJitter(0.1, 0.1, 0.1, 0.01),
        # transforms.RandomRotation(5),
        # transforms.RandomHorizontalFlip(),
        transforms.ToTensor(),
        transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
    ]),
    'val': transforms.Compose([
        transforms.Resize([vars.input_size, vars.input_size]),
        transforms.ToTensor(),
        transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
    ]),
    'test': transforms.Compose([
        transforms.Resize([vars.input_size, vars.input_size]),
        transforms.ToTensor(),
        transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
    ]),
}


# image_dataset_train = {'train': datasets.ImageFolder(os.path.join(vars.data_dir, 'train'), data_transforms['train'])}
# image_dataset_test = {'val': datasets.ImageFolder(os.path.join(vars.data_dir, 'val'), data_transforms['val'])}
# image_dataset_train.update(image_dataset_test)
# خط پایین با سه خط بالا برابری می کند!
#image_datasets = {x: datasets.ImageFolder(os.path.join(vars.data_dir, x), data_transforms[x])
                  #for x in ['train', 'val']}

image_datasets = {'train': datasets.ImageFolder(vars.train_dir, data_transforms['train']),
                  'val': datasets.ImageFolder(vars.val_dir, data_transforms['val']),
                  'test': datasets.ImageFolder(vars.test_dir, data_transforms['test'])}
#vars.dataloaders = {x: DataLoader(image_datasets[x], batch_size=vars.batch_size, shuffle=True, num_workers=0)
#              for x in ['train', 'val']}

test_ds = MIO_Dataset(image_datasets['test'].imgs, transform=data_transforms['test'])
test_dl = DataLoader(test_ds, batch_size=vars.batch_size, shuffle=False, num_workers=0)

#['articulated_truck', 'background', 'bicycle', 'bus', 'car', 'motorcycle', 'non-motorized_vehicle', 'pedestrian', 'pickup_truck', 'single_unit_truck', 'work_van']
#background: 160k, car: 260k, pickup_truck 50k, all others: less than 50k!
#weights = [1, 1/3, 1, 1, 1/4, 1, 1, 1, 1/2, 1, 1]
weights = [1/6, 1]#, 1/8, 1, 1/8]#, 1/4, 1/4, 1/4, 1, 1/4]
sampler = torch.utils.data.sampler.WeightedRandomSampler(weights, len(image_datasets['train']),)
#sampler = torch.utils.data.sampler.RandomSampler(len(image_datasets['train']),)

vars.dataloaders = {'train': DataLoader(image_datasets['train'], batch_size=vars.batch_size, shuffle=True, num_workers=0),
                    'val'  : DataLoader(image_datasets['val'], batch_size=vars.batch_size, shuffle=False, num_workers=0),
                    'test' : DataLoader(image_datasets['test'], batch_size=vars.batch_size, shuffle=False, num_workers=0),
                    #'test' : test_dl
                    }

vars.dataset_sizes = {x: len(image_datasets[x]) for x in ['train', 'val']}
vars.class_names = image_datasets['train'].classes

# Get a batch of training data
inputs, classes = next(iter(vars.dataloaders['train']))

# Make a grid from batch

#out = torchvision.utils.make_grid(inputs)

#utils.imshow(out, title=[vars.class_names[x] for x in classes])

print('Train Samples: ', vars.dataset_sizes['train'] )
print('Val Samples:   ', vars.dataset_sizes['val'] )
print('Model Name:    ', vars.model_name)
print('Batch Size:    ', vars.batch_size)
print('Learning Rate: ', vars.learning_rate)
######################################################################
# Finetuning the convnet
# Load a pretrained model and reset final fully connected layer.

##\\//\\//model, input_size = initialize_model(model_name, num_classes, feature_extract, use_pretrained=True)

if(vars.model_name.find('vgg') != -1):
    model = models.vgg11_bn(pretrained=vars.pre_trained)
    num_ftrs = model.classifier[6].in_features
    model.classifier[6] = nn.Linear(num_ftrs, len(vars.class_names))
elif(vars.model_name == 'resnet152'):
    model = models.resnet152(pretrained=vars.pre_trained)    
elif(vars.model_name == 'resnet50'):
    model = models.resnet50(pretrained=vars.pre_trained)
elif(vars.model_name == 'resnet18'):
    model = models.resnet18(pretrained=vars.pre_trained)
elif(vars.model_name == 'inception'):
        """ Inception v3
        Be careful, expects (299,299) sized images and has auxiliary output
        """
        model = models.inception_v3(pretrained=vars.pre_trained)
        # Handle the auxilary net
        num_ftrs = model.AuxLogits.fc.in_features
        model.AuxLogits.fc = nn.Linear(num_ftrs, len(vars.class_names))
# elif(vars.model_name == 'darknet53'):
#     model = darknet.darknet53(1000)
#     optimizer = optim.SGD(model.parameters(), lr = vars.learning_rate, momentum=0.9)
    # if(vars.pre_trained):
    #     #model.load_state_dict( torch.load('D:\\Projects\\_Python\\car Detection\\model_best.pth.tar') )    
    #     checkpoint = torch.load('D:\\Projects\\_Python\\car Detection\\model_best.pth.tar')
    #     start_epoch = checkpoint['epoch']
    #     best_acc1 = checkpoint['best_acc1']
    #     if vars.device.type == 'cuda':
    #         # best_acc1 may be from a checkpoint from a different GPU
    #         best_acc1 = best_acc1.to(vars.device)
    #     model.load_state_dict(checkpoint['state_dict'])
    #     optimizer.load_state_dict(checkpoint['optimizer'])
elif(vars.model_name == 'car3conv'):
    model = mymodels.car3conv()

elif(vars.model_name == 'car3conv'):
    model = mymodels.car3conv()
elif(vars.model_name == 'car5conv'):
    model = mymodels.car5conv()    
elif(vars.model_name == 'car2conv'):
    model = mymodels.car2conv()        
elif(vars.model_name == 'mymodel'):
    model = mymodels.MyModel()
elif(vars.model_name == 'myDensNet2'):
    model = mymodels.myDensNet(num_classes=len(vars.class_names))
elif(vars.model_name == 'myWCNN18'):
    model = mymodels.myWCNN18()
    #model.load_state_dict(torch.load("F:\\works\\Car Detection\\qovid-19\\myWCNN18-WGD_RAdam -cuda-batch-64\\ep50-acc99.30-loss0.0166.pth", map_location='cuda'))


# if(vars.model_name != 'vgg11'):
#     num_ftrs = model.fc.in_features
#     model.fc = nn.Linear(num_ftrs, len(vars.class_names))

model.to(vars.device)

if(vars.mode == 'test'):#test
    #model.load_state_dict(torch.load("D:\\Projects\\Python\\Zeitoon Detection\"))
    model.load_state_dict(torch.load(vars.test_model, map_location='cpu'))
    
    # log_dir = '{}-{}-{}-batch-{}'.format(vars.model_name, 'SGD', 'cuda', vars.batch_size)

    # if(vars.pre_trained):
    #     log_dir = log_dir + '-pretrained'
    # if not os.path.exists(log_dir):
    #     os.mkdir(log_dir)
    
    #from skimage.measure import compare_ssim as ssim
    #s = ssim(model.conv1.weight.data[1],model.conv1.weight.data[2], multichannel = True) 
    #Set multichannel False if black and white image.
    #print(s)

    log_file = open(".\\Time-{}-{}.log".format(vars.model_name, vars.batch_size),"w")    
    for dev in ['cuda']:# ['cuda']:
        vars.device = torch.device(dev)
        model = model.to(vars.device)
        model.eval()
        #run model on one batch to allocate required memory on device (and have more exact results)
        inputs = inputs.to(vars.device)
        outputs = model(inputs)

        #s = test_model_mio_tcd(model, vars.criterion, 'test', 1000000)
        s = test_model(model, vars.criterion, 'test', 100000)
        #ss = test_model_metric(model, vars.criterion, 'test', 100000)
        log_file.write(s)
        log_file.write('\n' + '-'*80)
    
    #log_file.write(str(summary(model, input_size=(3, vars.input_size, vars.input_size), batch_size=-1, device=vars.device.type)))
    log_file.close() 

    #summary(model, input_size=(3, vars.input_size, vars.input_size), batch_size=-1, device=vars.device.type)
    #print(summary(model, input_size=(3, vars.input_size, vars.input_size), batch_size=-1, device=vars.device.type))
else:
    
    #print(summary(model, input_size=(3, vars.input_size, vars.input_size), batch_size=-1, device=vars.device.type))
    #summary(model, input_size=(3, vars.input_size, vars.input_size), batch_size=-1, device=vars.device.type)
    #model.load_state_dict(torch.load("C:\\Projects\\Python\\Car Detection\\darknet53-SGD-cuda-batch-32\\ep7-acc97.83-loss0.0667.pth"))

    #optimizer = optim.SGD(model.parameters(), lr = vars.learning_rate , momentum=0.9, weight_decay = 0.0005)
    #optimizer = WGD_SGD(model.parameters(), lr = vars.learning_rate, momentum=0.9,weight_decay = 0.0005)
     
    #optimizer = optim.Adam(model.parameters(), vars.learning_rate, weight_decay = 0.0005) 
    #optimizer =WGD_Adam(model.parameters(), lr=vars.learning_rate, weight_decay = 0.0005)
     
    #optimizer = optim.Nadam(model.parameters(), vars.learning_rate, weight_decay = 0.0005)
    #optimizer = WGD_Nadam(model.parameters(), lr=vars.learning_rate, weight_decay = 0.0005)
    
    optimizer = RAdam(model.parameters(), lr=vars.learning_rate, weight_decay = 0.0005)
    #optimizer = WGD_RAdam(model.parameters(), vars.learning_rate, weight_decay = 0.0005) 
    
       
    #optimizer = optim.Adagrad(model.parameters(), lr=vars.learning_rate*0.1,weight_decay = 0.0005) 

    # Decay LR by a factor of 0.6 every 6 epochs
    exp_lr_scheduler = lr_scheduler.StepLR(optimizer, step_size = vars.scheduler_step_size, gamma = vars.scheduler_gamma)
    model = model.to(vars.device)
    model = train_model(model, vars.criterion, optimizer, exp_lr_scheduler, vars.num_epochs)
    visualize_model(model)  
    plt.ioff()
    plt.show()

# ######################################################################
# # ConvNet as fixed feature extractor
# # Here, we need to freeze all the network except the final layer. We need
# # to set ``requires_grad == False`` to freeze the parameters so that the
# # gradients are not computed in ``backward()``.
# # You can read more about this in the documentation
# # `here <https://pytorch.org/docs/notes/autograd.html#excluding-subgraphs-from-backward>`__.
# #

# model_conv = torchvision.models.resnet18(pretrained=True)
# for param in model_conv.parameters():
#     param.requires_grad = False

# # Parameters of newly constructed modules have requires_grad=True by default
# num_ftrs = model_conv.fc.in_features
# model_conv.fc = nn.Linear(num_ftrs, len(vars.class_names))

# model_conv = model_conv.to(vars.device)

# criterion = nn.CrossEntropyLoss()

# # Observe that only parameters of final layer are being optimized as
# # opposed to before.
# optimizer_conv = optim.SGD(model_conv.fc.parameters(), lr=0.001, momentum=0.9)

# # Decay LR by a factor of 0.1 every 7 epochs
# exp_lr_scheduler = lr_scheduler.StepLR(optimizer_conv, step_size=7, gamma=0.1)


# ######################################################################
# # Train and evaluate
# # ^^^^^^^^^^^^^^^^^^
# #
# # On CPU this will take about half the time compared to previous scenario.
# # This is expected as gradients don't need to be computed for most of the
# # network. However, forward does need to be computed.
# #

# model_conv = train_model(model_conv, criterion, optimizer_conv,
#                          exp_lr_scheduler, vars.num_epochs=25)

# ######################################################################
# #

# visualize_model(model_conv)


######################################################################
# checkout `Quantized Transfer Learning for Computer Vision Tutorial <https://pytorch.org/tutorials/intermediate/quantized_transfer_learning_tutorial.html>`_.
