import torch
import torch.nn as nn
import ops
from torch import optim
from torch.nn import functional as F
import torchvision
import mydensnets
#old model used in the first paper (car)
class car3conv(nn.Module):
    def __init__(self):
        super(car3conv, self).__init__()
        self.conv1 = nn.Conv2d(3, 16, kernel_size=3, padding=1) 
        self.bn1 = nn.BatchNorm2d(16)
        self.relu = nn.ReLU(inplace=True)
        self.pool = nn.MaxPool2d(4, 4)
        self.conv2 = nn.Conv2d(16, 32, 3, padding=1)
        self.pool2 = nn.MaxPool2d(2, 2)
        self.bn2 = nn.BatchNorm2d(32)
        self.conv3 = nn.Conv2d(32, 64, 3, padding=1)
        self.pool3 = nn.MaxPool2d(2, 2)
        self.bn3 = nn.BatchNorm2d(64)
        self.avgpool = nn.AdaptiveAvgPool2d((1, 1))
        self.fc1 = nn.Linear(64, 128)
        #self.fc2 = nn.Linear(64, 32)
        self.fc = nn.Linear(128, 6)
        #self.softmax = nn.Softmax()

    def forward(self, x):
        x = self.conv1(x)
        x = self.bn1(x)
        x = self.relu(x)
        x = self.pool(x)

        x = self.conv2(x)
        x = self.bn2(x)
        x = self.relu(x)
        x = self.pool2(x)

        x = self.conv3(x)
        x = self.bn3(x)
        x = self.relu(x)
        x = self.pool3(x)

        x = self.avgpool(x)
        x = torch.flatten(x, 1) #x = x.view(-1, 16 * 5 * 5)        
        x = self.relu(self.fc1(x))
        #x = self.relu(self.fc2(x))
        x = self.fc(x)
        #x = self.softmax(x)
        return x

class car5conv(nn.Module):
    def __init__(self):
        super(car5conv, self).__init__()
        self.conv1 = nn.Conv2d(3, 16, kernel_size=3, padding=1)
        self.bn1 = nn.BatchNorm2d(16)
        self.relu = nn.ReLU(inplace=True)
        self.pool = nn.MaxPool2d(2, 2) #128x128

        self.conv2 = nn.Conv2d(16, 32, 3, padding=1)
        self.bn2 = nn.BatchNorm2d(32)
        self.pool2 = nn.MaxPool2d(2, 2)#64x64
        
        self.conv3 = nn.Conv2d(32, 64, 3, padding=1)
        self.bn3 = nn.BatchNorm2d(64)

        self.pool3 = nn.MaxPool2d(2, 2)#32x32
        self.conv4 = nn.Conv2d(64, 64, 3, padding=1)
        self.bn4 = nn.BatchNorm2d(64)

        self.pool4 = nn.MaxPool2d(2, 2)#16x16
        self.conv5 = nn.Conv2d(64, 128, 3, padding=1)
        self.bn5 = nn.BatchNorm2d(128)

        self.pool5 = nn.MaxPool2d(2, 2)#8x8
        self.avgpool = nn.AdaptiveAvgPool2d((1, 1))
        self.fc1 = nn.Linear(128, 128)
        self.fc = nn.Linear(128, 6)

    def forward(self, x):
        x = self.conv1(x)
        x = self.bn1(x)
        x = self.relu(x)
        x = self.pool(x)

        x = self.conv2(x)
        x = self.bn2(x)
        x = self.relu(x)
        x = self.pool2(x)

        x = self.conv3(x)
        x = self.bn3(x)
        x = self.relu(x)
        x = self.pool3(x)

        x = self.conv4(x)
        x = self.bn4(x)
        x = self.relu(x)
        x = self.pool4(x)

        x = self.conv5(x)
        x = self.bn5(x)
        x = self.relu(x)
        x = self.pool5(x)

        x = self.avgpool(x)
        x = torch.flatten(x, 1) #x = x.view(-1, 16 * 5 * 5)        
        x = self.relu(self.fc1(x))
        x = self.fc(x)
        return x


#Batch normalization had bad effect on car2conv
class car2conv(nn.Module):
    def __init__(self):
        super(car2conv, self).__init__()
        self.conv1 = nn.Conv2d(3, 16, kernel_size=3, padding=1) 
        self.bn1 = nn.BatchNorm2d(16)
        self.relu = nn.ReLU(inplace=True)
        self.pool = nn.MaxPool2d(4, 4) #64x64
        self.conv2 = nn.Conv2d(16, 32, 3, padding=1)
        self.bn2 = nn.BatchNorm2d(32)
        self.pool2 = nn.MaxPool2d(4, 4) #32x32
        self.avgpool = nn.AdaptiveAvgPool2d((1, 1))
        self.fc1 = nn.Linear(32, 128)
        self.fc = nn.Linear(128, 6)

    def forward(self, x):
        x = self.conv1(x)
        #x = self.bn1(x)
        x = self.relu(x)
        x = self.pool(x)

        x = self.conv2(x)
        #x = self.bn2(x)
        x = self.relu(x)
        x = self.pool2(x)

        x = self.avgpool(x)
        x = torch.flatten(x, 1) #x = x.view(-1, 16 * 5 * 5)        
        x = self.relu(self.fc1(x))
        x = self.fc(x)
        return x


class MyModel(nn.Module):
    def __init__(self):
        super(MyModel, self).__init__()
        #self.conv1 = nn.Conv2d(3, 16, kernel_size=7, stride=2, padding=3, bias=False) #same as resnet except for number of filters (64) --> 128x128
        self.conv1 = nn.Conv2d(3, 16, kernel_size=3, padding=1) 
        self.bn1 = nn.BatchNorm2d(16)
        self.relu = nn.ReLU(inplace=True)
        #self.maxpool = nn.MaxPool2d(kernel_size=3, stride=2, padding=1)
        #self.conv1 = nn.Conv2d(3, 16, 3, padding=1)
        self.pool = nn.MaxPool2d(4, 4)
        self.conv2 = nn.Conv2d(16, 32, 3, padding=1)
        self.bn2 = nn.BatchNorm2d(32)
        self.conv3 = nn.Conv2d(32, 64, 3, padding=1)
        self.bn3 = nn.BatchNorm2d(64)
        self.avgpool = nn.AdaptiveAvgPool2d((1, 1))
        self.fc1 = nn.Linear(64, 128)
        #self.fc2 = nn.Linear(64, 32)
        self.fc = nn.Linear(128, 8)
        #self.softmax = nn.Softmax()

    def forward(self, x):
        x = self.conv1(x)
        x = self.bn1(x)
        x = self.relu(x)
        x = self.pool(x)

        x = self.conv2(x)
        x = self.bn2(x)
        x = self.relu(x)
        x = self.pool(x)

        x = self.conv3(x)
        x = self.bn3(x)
        x = self.relu(x)
        x = self.pool(x)

        x = self.avgpool(x)
        x = torch.flatten(x, 1) #x = x.view(-1, 16 * 5 * 5)        
        x = self.relu(self.fc1(x))
        #x = self.relu(self.fc2(x))
        x = self.fc(x)
        #x = self.softmax(x)
        return x


class MyModel2(nn.Module):
    def __init__(self):
        super(MyModel2, self).__init__()
        self.conv1 = nn.Conv2d(3, 16, kernel_size=7, stride=2, padding=3, bias=False) #same as resnet except for number of filters (64) --> 128x128
        #self.conv1 = nn.Conv2d(3, 16, kernel_size=3, padding=1) 
        self.bn1 = nn.BatchNorm2d(16)
        self.relu = nn.ReLU(inplace=True)
        self.pool = nn.MaxPool2d(2, 2)
        #self.conv2 = nn.Conv2d(16, 32, 3, padding=1)
        #self.bn2 = nn.BatchNorm2d(32)
        self.conv3 = nn.Conv2d(16, 64, 3, padding=1)
        self.bn3 = nn.BatchNorm2d(64)
        self.avgpool = nn.AdaptiveAvgPool2d((1, 1))
        self.fc1 = nn.Linear(64, 128)
        #self.fc2 = nn.Linear(64, 32)
        self.fc = nn.Linear(128, 8)

    def forward(self, x):
        x = self.conv1(x)
        x = self.bn1(x)
        x = self.relu(x)
        x = self.pool(x)

        x = self.conv3(x)
        x = self.bn3(x)
        x = self.relu(x)
        x = self.pool(x)

        x = self.avgpool(x)
        x = torch.flatten(x, 1) #x = x.view(-1, 16 * 5 * 5)        
        x = self.relu(self.fc1(x))
        #x = self.relu(self.fc2(x))
        x = self.fc(x)
        #x = self.softmax(x)
        return x
# Squeeze and Excitation Block Module
class SEBlock(nn.Module):
    def __init__(self, channels, reduction=16):
        super(SEBlock, self).__init__()

        self.fc = torch.nn.Sequential(
            torch.nn.Conv2d(channels, channels // reduction, 1, bias=False),
            torch.nn.ReLU(),
            torch.nn.Conv2d(channels // reduction, channels * 2, 1, bias=False),
        )

    def forward(self, x):
        w = torch.nn.functional.adaptive_avg_pool2d(x, 1) # Squeeze
        w = self.fc(x)
        w, b = w.split(w.data.size(1) // 2, dim=1) # Excitation
        w = torch.sigmoid(w)

        return x * w + b # Scale and add bias


class DWT(nn.Module):
    def __init__(self):
        super(DWT, self).__init__()
        self.requires_grad = False

    def forward(self, x):
        return ops.dwt_init(x)

class IWT(nn.Module):
    def __init__(self):
        super(IWT, self).__init__()
        self.requires_grad = False

    def forward(self, x):
        return ops.iwt_init(x)

class DWT_4out(nn.Module):
    def __init__(self):
        super(DWT_4out, self).__init__()
        self.requires_grad = False

    def forward(self, x):
        return ops.dwt_init_4out(x)

class myWCNN(nn.Module):
    def __init__(self):
        super(myWCNN, self).__init__()
        self.DWT = DWT()
        self.layer1 = nn.Sequential(
            nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2),
            nn.MaxPool2d(kernel_size=2, stride=2, padding=0),
            nn.BatchNorm2d(32),
            nn.ReLU())
        self.layer2 = nn.Sequential(
            nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2),
            nn.MaxPool2d(kernel_size=2, stride=2, padding=0),
            nn.BatchNorm2d(32),
            nn.ReLU())
        self.layer3 = nn.Sequential(
            nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2),
            nn.MaxPool2d(kernel_size=2, stride=2, padding=0),
            nn.BatchNorm2d(64),
            nn.ReLU())
        self.se_block1 = SEBlock(64)
        self.layer4 = nn.Sequential(
            nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2),
            nn.MaxPool2d(kernel_size=2, stride=2, padding=0),
            nn.BatchNorm2d(128),
            nn.ReLU())
        self.se_block2 = SEBlock(128)
        self.layer5 = nn.Sequential(
            nn.Conv2d(128, 256, kernel_size=5, stride=1, padding=2),
            nn.MaxPool2d(kernel_size=2, stride=2, padding=0),
            nn.BatchNorm2d(256),
            nn.ReLU())
        self.se_block3 = SEBlock(256)
        self.drop_out = nn.Dropout()
        self.fc1 = nn.Linear(8 * 8 * 128, 128)#*128,256)
        self.fc = nn.Linear(128, 5)#(256, 5)

    def forward(self, x):
        out = self.layer1(x)
        out = self.layer2(out)
        out = self.layer3(out)
        out = self.se_block1(out)
        out = self.layer4(out)
        out = self.se_block2(self.DWT(out))
        #out = self.layer5(out)
        #out = self.se_block3(self.DWT(out))
        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out

class myWCNN2(nn.Module):
    def __init__(self):
        super(myWCNN2, self).__init__()
        self.DWT = DWT()
        self.layer1 = nn.Sequential(
            nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2),
            nn.MaxPool2d(kernel_size=2, stride=2, padding=0),
            nn.BatchNorm2d(32),
            nn.ReLU())
        self.layer2 = nn.Sequential(
            nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2),
            nn.MaxPool2d(kernel_size=2, stride=2, padding=0),
            nn.BatchNorm2d(32),
            nn.ReLU())
        self.layer3 = nn.Sequential(
            nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2),
            nn.MaxPool2d(kernel_size=2, stride=2, padding=0),
            nn.BatchNorm2d(64),
            nn.ReLU())
        self.se_block1 = SEBlock(64)
        self.layer4 = nn.Sequential(
            nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2),
            nn.MaxPool2d(kernel_size=2, stride=2, padding=0),
            nn.BatchNorm2d(128),
            nn.ReLU())
        self.se_block2 = SEBlock(128)
        self.layer5 = nn.Sequential(
            nn.Conv2d(128, 128, kernel_size=5, stride=1, padding=2),
            nn.MaxPool2d(kernel_size=2, stride=2, padding=0),
            nn.BatchNorm2d(128),
            nn.ReLU())
        self.se_block3 = SEBlock(256)
        self.drop_out = nn.Dropout()
        self.fc1 = nn.Linear(8 * 8 * 128, 128)#*128,256)
        self.fc = nn.Linear(128, 5)#(256, 5)

    def forward(self, x):
        out = self.layer1(x)
        out = self.layer2(out)
        out = self.layer3(out)
        out = self.se_block1(out)
        out = self.layer4(out)
        #out = self.se_block2(self.DWT(out))
        out = self.se_block2(out)
        out = self.layer5(out)
        #out = self.se_block3(self.DWT(out))
        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out

class myWCNN3(nn.Module):
    def __init__(self):
        super(myWCNN3, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2)
        self.pool1 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2)
        self.pool2 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2)
        self.pool3 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2)
        self.pool4 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=5, stride=1, padding=2)
        self.pool5 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 11)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.DWT(out)
        out = self.bn1(out)
        out = self.re1(out)

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.DWT(out)
        out = self.bn2(out)
        out = self.re2(out)

        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.DWT(out)
        out = self.bn3(out)
        out = self.re3(out)

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.DWT(out)
        out = self.bn4(out)
        out = self.re4(out)

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.DWT(out)
        out = self.bn5(out)
        out = self.re5(out)

        out = self.se_block5(out)

        # out = self.se_block1(out)
        # out = self.layer4(out)
        # out = self.se_block2(self.DWT(out))
        # #out = self.se_block2(out)
        # out = self.layer5(out)
        # #out = self.se_block3(self.DWT(out))
        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out

class myWCNN4(nn.Module):
    def __init__(self):
        super(myWCNN4, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2)
        self.pool1 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2)
        self.pool2 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2)
        self.pool3 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2)
        self.pool4 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=5, stride=1, padding=2)
        self.pool5 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 5)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.DWT(out)
        out = self.bn1(out)
        out = self.re1(out)

        #out = self.se_block1(out)

        out = self.conv2(out)
        out = self.DWT(out)
        out = self.bn2(out)
        out = self.re2(out)

        #out = self.se_block2(out)

        out = self.conv3(out)
        out = self.DWT(out)
        out = self.bn3(out)
        out = self.re3(out)

       # out = self.se_block3(out)

        out = self.conv4(out)
        out = self.DWT(out)
        out = self.bn4(out)
        out = self.re4(out)

       # out = self.se_block4(out)

        out = self.conv5(out)
        out = self.DWT(out)
        out = self.bn5(out)
        out = self.re5(out)

       # out = self.se_block5(out)


        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out

class myWCNN5(nn.Module):
    def __init__(self):
        super(myWCNN5, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2)
        self.pool1 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2)
        self.pool2 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2)
        self.pool3 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2)
        self.pool4 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=5, stride=1, padding=2)
        self.pool5 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 5)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.pool1(out)
        out = self.bn1(out)
        out = self.re1(out)

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.pool2(out)
        out = self.bn2(out)
        out = self.re2(out)

        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.pool3(out)
        out = self.bn3(out)
        out = self.re3(out)

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.pool4(out)
        out = self.bn4(out)
        out = self.re4(out)

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.pool5(out)
        out = self.bn5(out)
        out = self.re5(out)

        out = self.se_block5(out)

        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out

class myWCNN6(nn.Module):
    def __init__(self):
        super(myWCNN6, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2)
        self.pool1 = nn.AvgPool2d(kernel_size=2, stride=2, padding=0) 
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2)
        self.pool2 = nn.AvgPool2d(kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2)
        self.pool3 = nn.AvgPool2d(kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2)
        self.pool4 = nn.AvgPool2d(kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=5, stride=1, padding=2)
        self.pool5 = nn.AvgPool2d(kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 5)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.pool1(out)
        out = self.bn1(out)
        out = self.re1(out)

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.pool2(out)
        out = self.bn2(out)
        out = self.re2(out)

        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.pool3(out)
        out = self.bn3(out)
        out = self.re3(out)

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.pool4(out)
        out = self.bn4(out)
        out = self.re4(out)

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.pool5(out)
        out = self.bn5(out)
        out = self.re5(out)

        out = self.se_block5(out)

        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out

class myWCNN7(nn.Module):
    def __init__(self):
        super(myWCNN7, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2)
        self.pool1 = nn.MaxPool2d( kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2)
        self.pool2 = nn.MaxPool2d( kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2)
        self.pool3 = nn.MaxPool2d( kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2)
        self.pool4 = nn.MaxPool2d( kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=5, stride=1, padding=2)
        self.pool5 = nn.MaxPool2d( kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.globalavgpool = nn.AdaptiveAvgPool2d((1, 1))
        self.fc1 = nn.Linear(256, 256)#8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 5)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.pool1(out)
        out = self.bn1(out)
        out = self.re1(out)

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.pool2(out)
        out = self.bn2(out)
        out = self.re2(out)

        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.pool3(out)
        out = self.bn3(out)
        out = self.re3(out)

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.pool4(out)
        out = self.bn4(out)
        out = self.re4(out)

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.pool5(out)
        out = self.bn5(out)
        out = self.re5(out)

        out = self.se_block5(out)
        out = self.globalavgpool(out)
        out = torch.flatten(out, 1)
        #out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        
        out = self.fc1(out)
        out = self.fc(out)
        return out

class myWCNN8(nn.Module):
    def __init__(self):
        super(myWCNN8, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2)
        self.pool1 = nn.LPPool2d(2, kernel_size=2, stride=2)#, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2)
        self.pool2 = nn.LPPool2d(2, kernel_size=2, stride=2)#, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2)
        self.pool3 = nn.LPPool2d(2, kernel_size=2, stride=2)#, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2)
        self.pool4 = nn.LPPool2d(2, kernel_size=2, stride=2)#, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=5, stride=1, padding=2)
        self.pool5 = nn.LPPool2d(2, kernel_size=2, stride=2)#, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 5)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.pool1(out)
        out = self.bn1(out)
        out = self.re1(out)

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.pool2(out)
        out = self.bn2(out)
        out = self.re2(out)

        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.pool3(out)
        out = self.bn3(out)
        out = self.re3(out)

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.pool4(out)
        out = self.bn4(out)
        out = self.re4(out)

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.pool5(out)
        out = self.bn5(out)
        out = self.re5(out)

        out = self.se_block5(out)


        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out

class myWCNN9(nn.Module):
    def __init__(self):
        super(myWCNN9, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2)
        self.pool1 = nn.MaxPool2d( kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2)
        self.pool2 = nn.MaxPool2d( kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2)
        self.pool3 = nn.MaxPool2d( kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2)
        self.pool4 = nn.MaxPool2d( kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=5, stride=1, padding=2)
        self.pool5 = nn.MaxPool2d( kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.globalavgpool = nn.AdaptiveAvgPool2d((1, 1))
        self.fc1 = nn.Linear(256, 256)#8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 5)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.DWT(out)
        out = self.bn1(out)
        out = self.re1(out)

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.DWT(out)
        out = self.bn2(out)
        out = self.re2(out)

        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.DWT(out)
        out = self.bn3(out)
        out = self.re3(out)

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.DWT(out)
        out = self.bn4(out)
        out = self.re4(out)

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.DWT(out)
        out = self.bn5(out)
        out = self.re5(out)

        out = self.se_block5(out)
        out = self.globalavgpool(out)
        out = torch.flatten(out, 1)
        #out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        
        out = self.fc1(out)
        out = self.fc(out)
        return out

class myWCNN10(nn.Module):
    def __init__(self):
        super(myWCNN10, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2)
        self.pool1 = nn.AvgPool2d( kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2)
        self.pool2 = nn.AvgPool2d( kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2)
        self.pool3 = nn.AvgPool2d( kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2)
        self.pool4 = nn.AvgPool2d( kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=5, stride=1, padding=2)
        self.pool5 = nn.AvgPool2d( kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.globalavgpool = nn.AdaptiveAvgPool2d((1, 1))
        self.fc1 = nn.Linear(256, 256)#8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 5)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.pool1(out)
        out = self.bn1(out)
        out = self.re1(out)

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.pool2(out)
        out = self.bn2(out)
        out = self.re2(out)

        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.pool3(out)
        out = self.bn3(out)
        out = self.re3(out)

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.pool4(out)
        out = self.bn4(out)
        out = self.re4(out)

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.pool5(out)
        out = self.bn5(out)
        out = self.re5(out)

        out = self.se_block5(out)
        out = self.globalavgpool(out)
        out = torch.flatten(out, 1)
        #out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        
        out = self.fc1(out)
        out = self.fc(out)
        return out

class myWCNN11(nn.Module):
    def __init__(self):
        super(myWCNN11, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2)
        self.pool1 = nn.LPPool2d(2, kernel_size=2, stride=2)#, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2)
        self.pool2 = nn.LPPool2d(2, kernel_size=2, stride=2)#, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2)
        self.pool3 = nn.LPPool2d(2, kernel_size=2, stride=2)#, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2)
        self.pool4 = nn.LPPool2d(2, kernel_size=2, stride=2)#, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=5, stride=1, padding=2)
        self.pool5 = nn.LPPool2d(2, kernel_size=2, stride=2)#, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.globalavgpool = nn.AdaptiveAvgPool2d((1, 1))
        self.fc1 = nn.Linear(256, 256)#8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 5)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.pool1(out)
        out = self.bn1(out)
        out = self.re1(out)

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.pool2(out)
        out = self.bn2(out)
        out = self.re2(out)

        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.pool3(out)
        out = self.bn3(out)
        out = self.re3(out)

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.pool4(out)
        out = self.bn4(out)
        out = self.re4(out)

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.pool5(out)
        out = self.bn5(out)
        out = self.re5(out)

        out = self.se_block5(out)

        out = self.globalavgpool(out)
        out = torch.flatten(out, 1)
        #out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        
        out = self.fc1(out)
        out = self.fc(out)
        return out

class myWCNN13(nn.Module):
    def __init__(self):
        super(myWCNN13, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2)
        self.pool1 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2)
        self.pool2 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2)
        self.pool3 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2)
        self.pool4 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=5, stride=1, padding=2)
        self.pool5 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.drop_out_in = nn.Dropout(p=0.1)
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 11)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.re1(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn1(out)
       

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.re2(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn2(out)


        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.re3(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn3(out)
        

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.re4(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn4(out)
        

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.re5(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn5(out)
        

        out = self.se_block5(out)

        # out = self.se_block1(out)
        # out = self.layer4(out)
        # out = self.se_block2(self.DWT(out))
        # #out = self.se_block2(out)
        # out = self.layer5(out)
        # #out = self.se_block3(self.DWT(out))
        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out


class myWCNN14(nn.Module):
    def __init__(self):
        super(myWCNN14, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=5, stride=1, padding=2)
        self.pool1 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=5, stride=1, padding=2)
        self.pool2 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=5, stride=1, padding=2)
        self.pool3 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=5, stride=1, padding=2)
        self.pool4 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=5, stride=1, padding=2)
        self.pool5 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.drop_out_in = nn.Dropout2d(p=0.1)
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 11)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.re1(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn1(out)
       

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.re2(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn2(out)


        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.re3(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn3(out)
        

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.re4(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn4(out)
        

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.re5(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn5(out)
        

        out = self.se_block5(out)

        # out = self.se_block1(out)
        # out = self.layer4(out)
        # out = self.se_block2(self.DWT(out))
        # #out = self.se_block2(out)
        # out = self.layer5(out)
        # #out = self.se_block3(self.DWT(out))
        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out


class myWCNN15(nn.Module):
    def __init__(self):
        super(myWCNN15, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=3, stride=1, padding=1)
        self.pool1 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=3, stride=1, padding=1)
        self.pool2 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=3, stride=1, padding=1)
        self.pool3 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=3, stride=1, padding=1)
        self.pool4 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=3, stride=1, padding=1)
        self.pool5 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.drop_out_in = nn.Dropout2d(p=0.1)
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 11)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.re1(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn1(out)
       

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.re2(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn2(out)


        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.re3(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn3(out)
        

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.re4(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn4(out)
        

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.re5(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn5(out)
        

        out = self.se_block5(out)

        # out = self.se_block1(out)
        # out = self.layer4(out)
        # out = self.se_block2(self.DWT(out))
        # #out = self.se_block2(out)
        # out = self.layer5(out)
        # #out = self.se_block3(self.DWT(out))
        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out


class myWCNN16(nn.Module):
    def __init__(self):
        super(myWCNN16, self).__init__()
        self.DWT = DWT()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=3, stride=1, padding=1)
        self.pool1 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=3, stride=1, padding=1)
        self.pool2 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=3, stride=1, padding=1)
        self.pool3 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=3, stride=1, padding=1)
        self.pool4 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=3, stride=1, padding=1)
        self.pool5 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.drop_out_in = nn.Dropout(p=0.1)
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 11)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.re1(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn1(out)
       

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.re2(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn2(out)


        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.re3(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn3(out)
        

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.re4(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn4(out)
        

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.re5(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn5(out)
        

        out = self.se_block5(out)

        # out = self.se_block1(out)
        # out = self.layer4(out)
        # out = self.se_block2(self.DWT(out))
        # #out = self.se_block2(out)
        # out = self.layer5(out)
        # #out = self.se_block3(self.DWT(out))
        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out

#mish activation function    
class Mish(nn.Module):
    
    def __init__(self):
        super().__init__()
        print("Mish activation loaded...")

    def forward(self, x): 
        
        x = x *( torch.tanh(F.softplus(x)))

        return x


class myWCNN17(nn.Module):
    def __init__(self):
        super(myWCNN17, self).__init__()
        self.DWT = DWT()
        self.Mish = Mish()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=3, stride=1, padding=1)
        self.pool1 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=3, stride=1, padding=1)
        self.pool2 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=3, stride=1, padding=1)
        self.pool3 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=3, stride=1, padding=1)
        self.pool4 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=3, stride=1, padding=1)
        self.pool5 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.drop_out_in = nn.Dropout(p=0.1)
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 11)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn1(out)
       

        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn2(out)


        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn3(out)
        

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn4(out)
        

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.drop_out_in(out)
        out = self.bn5(out)
        

        out = self.se_block5(out)

        # out = self.se_block1(out)
        # out = self.layer4(out)
        # out = self.se_block2(self.DWT(out))
        # #out = self.se_block2(out)
        # out = self.layer5(out)
        # #out = self.se_block3(self.DWT(out))
        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out


#myWCNN17 with changing the batchnormalization place
class myWCNN18(nn.Module):
    def __init__(self):
        super(myWCNN18, self).__init__()
        self.DWT = DWT()
        self.Mish = Mish()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=3, stride=1, padding=1)
        self.pool1 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=3, stride=1, padding=1)
        self.pool2 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=3, stride=1, padding=1)
        self.pool3 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=3, stride=1, padding=1)
        self.pool4 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=3, stride=1, padding=1)
        self.pool5 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.drop_out_in = nn.Dropout(p=0.1)
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 2)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.bn1(out)
        out = self.drop_out_in(out)
        
       
        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.bn2(out)
        out = self.drop_out_in(out)
        
        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.bn3(out)
        out = self.drop_out_in(out)
        
        

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.bn4(out)
        out = self.drop_out_in(out)
       
        

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.bn5(out)
        out = self.drop_out_in(out)
        
        

        out = self.se_block5(out)

        # out = self.se_block1(out)
        # out = self.layer4(out)
        # out = self.se_block2(self.DWT(out))
        # #out = self.se_block2(out)
        # out = self.layer5(out)
        # #out = self.se_block3(self.DWT(out))
        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out


#myWCNN18 with 4 output in wavelet
class myWCNN19(nn.Module):
    def __init__(self):
        super(myWCNN19, self).__init__()
        self.DWT = DWT_4out()
        self.Mish = Mish()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=3, stride=1, padding=1)
        self.pool1 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=3, stride=1, padding=1)
        self.pool2 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=3, stride=1, padding=1)
        self.pool3 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=3, stride=1, padding=1)
        self.pool4 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=3, stride=1, padding=1)
        self.pool5 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.drop_out_in = nn.Dropout(p=0.1)
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 11)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.bn1(out)
        out = self.drop_out_in(out)
        
       
        out = self.se_block1(out)

        out = self.conv2(out)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.bn2(out)
        out = self.drop_out_in(out)
        
        out = self.se_block2(out)

        out = self.conv3(out)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.bn3(out)
        out = self.drop_out_in(out)
        
        

        out = self.se_block3(out)

        out = self.conv4(out)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.bn4(out)
        out = self.drop_out_in(out)
       
        

        out = self.se_block4(out)

        out = self.conv5(out)
        out = self.Mish(out)
        out = self.DWT(out)
        out = self.bn5(out)
        out = self.drop_out_in(out)
        
        

        out = self.se_block5(out)

        # out = self.se_block1(out)
        # out = self.layer4(out)
        # out = self.se_block2(self.DWT(out))
        # #out = self.se_block2(out)
        # out = self.layer5(out)
        # #out = self.se_block3(self.DWT(out))
        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out

#myWCNN18 with maxpooling without seblock
class myWCNN20(nn.Module):
    def __init__(self):
        super(myWCNN20, self).__init__()
        self.DWT = DWT()
        self.Mish = Mish()
        
        self.conv1 = nn.Conv2d(3, 32, kernel_size=3, stride=1, padding=1)
        self.pool1 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn1 = nn.BatchNorm2d(32)
        self.re1 = nn.ReLU()
        
        self.se_block1 = SEBlock(32)

        self.conv2 = nn.Conv2d(32, 32, kernel_size=3, stride=1, padding=1)
        self.pool2 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn2 = nn.BatchNorm2d(32)
        self.re2 = nn.ReLU()

        self.se_block2 = SEBlock(32)
        
        self.conv3 = nn.Conv2d(32, 64, kernel_size=3, stride=1, padding=1)
        self.pool3 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn3 = nn.BatchNorm2d(64)
        self.re3 = nn.ReLU()

        self.se_block3 = SEBlock(64)
        
        self.conv4 = nn.Conv2d(64, 128, kernel_size=3, stride=1, padding=1)
        self.pool4 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn4 = nn.BatchNorm2d(128)
        self.re4 = nn.ReLU()

        self.se_block4 = SEBlock(128)
        
        self.conv5 = nn.Conv2d(128, 256, kernel_size=3, stride=1, padding=1)
        self.pool5 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        self.bn5 = nn.BatchNorm2d(256)
        self.re5 = nn.ReLU()

        self.se_block5 = SEBlock(256)

        self.drop_out = nn.Dropout()
        self.drop_out_in = nn.Dropout(p=0.1)
        self.fc1 = nn.Linear(8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(256, 11)#(256, 5)

    def forward(self, x):
        out = self.conv1(x)
        out = self.Mish(out)
        out = self.pool1(out)
        out = self.bn1(out)
        out = self.drop_out_in(out)
        
       
        #out = self.se_block1(out)

        out = self.conv2(out)
        out = self.Mish(out)
        out = self.pool2(out)
        out = self.bn2(out)
        out = self.drop_out_in(out)
        
        #out = self.se_block2(out)

        out = self.conv3(out)
        out = self.Mish(out)
        out = self.pool3(out)
        out = self.bn3(out)
        out = self.drop_out_in(out)
        
        

        #out = self.se_block3(out)

        out = self.conv4(out)
        out = self.Mish(out)
        out = self.pool4(out)
        out = self.bn4(out)
        out = self.drop_out_in(out)
       
        

        #out = self.se_block4(out)

        out = self.conv5(out)
        out = self.Mish(out)
        out = self.pool5(out)
        out = self.bn5(out)
        out = self.drop_out_in(out)
        
        

        #out = self.se_block5(out)

        # out = self.se_block1(out)
        # out = self.layer4(out)
        # out = self.se_block2(self.DWT(out))
        # #out = self.se_block2(out)
        # out = self.layer5(out)
        # #out = self.se_block3(self.DWT(out))
        out = out.reshape(out.size(0), -1)
        out = self.drop_out(out)
        out = self.fc1(out)
        out = self.fc(out)
        return out


#######@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

####### New Networks


class FlattenLayer(nn.Module):
    def __init__(self):
        super(FlattenLayer, self).__init__()

    def forward(self, X):
        return X.view(X.shape[0], -1)

class GlobalAvgPool2d(nn.Module):
    def __init__(self):
        super(GlobalAvgPool2d, self).__init__()

    def forward(self, x):
        return F.avg_pool2d(x, kernel_size=x.size()[2:])

class Inception(nn.Module):
    # c1-c4为四条线路输出的通道数
    def __init__(self, in_c, c1, c2, c3, c4):
        super(Inception, self).__init__()
        self.p1_1 = nn.Conv2d(in_c, c1, kernel_size=1)

        self.p2_1 = nn.Conv2d(in_c, c2[0], 1)
        self.p2_2 = nn.Conv2d(c2[0], c2[1], kernel_size=3, padding=1)

        self.p3_1 = nn.Conv2d(in_c, c3[0], kernel_size=1)
        self.p3_2 = nn.Conv2d(c3[0], c3[1], kernel_size=5, padding=2)

        self.p4_1 = nn.MaxPool2d(kernel_size=3, stride=1, padding=1)
        self.p4_2 = nn.Conv2d(in_c, c4, kernel_size=1)

    def forward(self, x):
        p1 = F.relu(self.p1_1(x))
        p2 = F.relu(self.p2_2(F.relu((self.p2_1(x)))))
        p3 = F.relu(self.p3_2(F.relu(self.p3_1(x))))
        p4 = F.relu(self.p4_2(self.p4_1(x)))
        return torch.cat((p1, p2, p3, p4), dim=1)


class GoogleNet(nn.Module):
    def __init__(self):
        
        super(GoogleNet, self).__init__()
        self.b1 = nn.Sequential(
            nn.Conv2d(3, 64, kernel_size=7, stride=2, padding=3),
            nn.ReLU(),
            nn.MaxPool2d(kernel_size=3, stride=2, padding=1)
        )
        self.b2 = nn.Sequential(
            nn.Conv2d(64, 64, kernel_size=1),
            nn.Conv2d(64, 192, kernel_size=3, padding=1),
            nn.ReLU(),
            nn.MaxPool2d(kernel_size=3, stride=2, padding=1)
        )
        self.b3 = nn.Sequential(
            Inception(192, 64, (96, 128), (16, 32), 32),
            Inception(256, 128, (128, 192), (32, 96), 64),
            nn.MaxPool2d(kernel_size=3, stride=2, padding=1)
        )
        self.b4 = nn.Sequential(
            Inception(480, 192, (96, 208), (16, 48), 64),
            Inception(512, 160, (112, 224), (24, 64), 64),
            Inception(512, 128, (128, 256), (24, 64), 64),
            Inception(512, 112, (144, 288), (32, 64), 64),
            Inception(528, 256, (160, 320), (32, 128), 128),
            nn.MaxPool2d(kernel_size=3, stride=2, padding=1)
        )
        self.b5 = nn.Sequential(
            Inception(832, 256, (160, 320), (32, 128), 128),
            Inception(832, 384, (192, 384), (48, 128), 128),
           # GlobalAvgPool2d(),
            # nn.AdaptiveAvgPool2d((1, 1)),
            # nn.Dropout(0.4),
            # FlattenLayer(),
            # nn.Linear(1024, 5)
        )
        self.net = nn.Sequential(self.b1, self.b2, self.b3, self.b4, self.b5)
        self.gap = nn.AdaptiveAvgPool2d((1, 1))
        self.drop = nn.Dropout(0.4)

        self.fc = nn.Linear(1024,5)
    def forward(self, x):
        out = self.net(x)
        out = self.gap(out)
        out = self.drop(out)
        out = torch.flatten(out, 1)
        out = self.fc(out)
        return out

class NIN(nn.Module):
    def __init__(self):
        
        super(NIN, self).__init__()
       # self.net = nn.Sequential(
        self.nin_bl1 = ops.nin_block(3, 96, kernel_size=11, stride=4, padding=0)
        self.pool1 = nn.MaxPool2d(kernel_size=3, stride=2)
        self.nin_bl2 = ops.nin_block(96, 256, kernel_size=5, stride=1, padding=2)
        self.pool2 = nn.MaxPool2d(kernel_size=3, stride=2)
        self.nin_bl3 = ops.nin_block(256, 384, kernel_size=3, stride=1, padding=1)
        self.pool3 = nn.MaxPool2d(kernel_size=3, stride=2)
        self.drop1 = nn.Dropout(0.5)
        self.nin_bl4 = ops.nin_block(384, 5, kernel_size=3, stride=1, padding=1)
        #self.gap = GlobalAvgPool2d()
        self.gap = nn.AdaptiveAvgPool2d((1, 1))
           # FlattenLayer()
        self.fc1 = nn.Linear(384, 384)#8 * 8 * 256, 256)#*128,256)
        self.fc = nn.Linear(5, 2)#(256, 5)
        

    # def nin_block(self, in_channels, out_channels, kernel_size, stride, padding):
    #     block = nn.Sequential(
    #         nn.Conv2d(in_channels, out_channels, kernel_size, stride, padding),
    #         nn.ReLU(),
    #         nn.Conv2d(out_channels, out_channels, kernel_size=1),
    #         nn.ReLU(),
    #         nn.Conv2d(out_channels, out_channels, kernel_size=1),
    #         nn.ReLU()
    #     )
    #     return block

    def forward(self, x):
        out = self.nin_bl1(x)
        out = self.pool1(out)
        out = self.nin_bl2(out)
        out = self.pool2(out)
        out = self.nin_bl3(out)
        out = self.pool3(out)
        out = self.drop1(out)
        out = self.nin_bl4(out)
        out = self.gap(out)
        out = torch.flatten(out, 1)
        #out = self.fc1(out)
        out = self.fc(out)
        return out

class LeNet(nn.Module):
    def __init__(self):
        super(LeNet, self).__init__()
        self.conv = nn.Sequential(
            nn.Conv2d(1, 6, 5),
            nn.Sigmoid(),
            nn.AvgPool2d(2, 2),
            nn.Conv2d(6, 16, 5),
            nn.Sigmoid(),
            nn.AvgPool2d(2, 2)
        )
        self.fc = nn.Sequential(
            nn.Linear(16 * 4 * 4, 120),
            nn.Sigmoid(),
            nn.Linear(120, 84),
            nn.Sigmoid(),
            nn.Linear(84, 5)
        )

    def forward(self, img):
        feature = self.conv(img)
        output = self.fc(feature.view(feature.shape[0], -1))
        return output

class weightconv(nn.Module):
    def __init__(self):
        super(weightconv, self).__init__()
        self.conv1 = nn.Conv2d(3, 5, kernel_size=3, stride=1, padding=1)
        self.pool1 = nn.MaxPool2d(kernel_size=2, stride=2, padding=0)
        #self.bn1 = nn.BatchNorm2d(5)
        self.re1 = nn.ReLU()
        # self.conv2 = nn.Conv2d(16, 32, 3, padding=1)
        # self.pool2 = nn.MaxPool2d(2, 2)
        # self.bn2 = nn.BatchNorm2d(32)
        # self.conv3 = nn.Conv2d(32, 64, 3, padding=1)
        # self.pool3 = nn.MaxPool2d(2, 2)
        # self.bn3 = nn.BatchNorm2d(64)
        self.fc1 = nn.Linear(5 * 128 * 128, 256)#*128,256)
        self.fc = nn.Linear(256, 5)#(256, 5)
        #self.softmax = nn.Softmax()

    def forward(self, x):
        out = self.conv1(x)
        #out = self.bn1(x)
        out = self.re1(out)
        out = self.pool1(out)

        # x = self.conv2(x)
        # x = self.bn2(x)
        # x = self.relu(x)
        # x = self.pool2(x)

        # x = self.conv3(x)
        # x = self.bn3(x)
        # x = self.relu(x)
        # x = self.pool3(x)

        out = out.reshape(out.size(0), -1)
        
        out = self.fc1(out)
        out = self.fc(out)
        return out

class myDensNet(nn.Module):
    def __init__(self, num_classes=1000, num_channels=3):
        super(myDensNet, self).__init__()
        preloaded = torchvision.models.densenet121(pretrained=False)
        self.features = preloaded.features
        self.features.conv0 = nn.Conv2d(num_channels, 64, 7, 2, 3)
        self.classifier = nn.Linear(1024, num_classes, bias=True)
        del preloaded
        
    def forward(self, x):
        features = self.features(x)
        out = F.relu(features, inplace=True)
        out = F.adaptive_avg_pool2d(out, (1, 1)).view(features.size(0), -1)
        out = self.classifier(out)
        return out

#densenet121 with mish
class myDensNet2(nn.Module):
    def __init__(self, num_classes=1000, num_channels=3):
        super(myDensNet2, self).__init__()
        preloaded = mydensnets.densenet121(pretrained=False)
        self.features = preloaded.features
        self.features.conv0 = nn.Conv2d(num_channels, 64, 7, 2, 3)
        self.classifier = nn.Linear(1024, num_classes, bias=True)
        del preloaded
        
    def forward(self, x):
        features = self.features(x)
        out = F.relu(features, inplace=True)
        out = F.adaptive_avg_pool2d(out, (1, 1)).view(features.size(0), -1)
        out = self.classifier(out)
        return out


