import matplotlib.pyplot as plt
from sklearn.metrics import classification_report, confusion_matrix
import numpy as np
import vars
def plot_confusion_matrix(cm, 
                          normalize=False,
                          title=None,
                          cmap=plt.cm.Blues):
    """
    This function prints and plots the confusion matrix.
    Normalization can be applied by setting `normalize=True`.
    """
    if not title:
        if normalize:
            title = 'Normalized confusion matrix'
        else:
            title = 'Confusion matrix, without normalization'

    # Only use the labels that appear in the data
    #classes = classes[unique_labels(y_true, y_pred)]
    ncm = cm.astype('float') / cm.sum(axis=1)[:, np.newaxis]
    if normalize:
        cm = ncm
        print("Normalized confusion matrix")
    else:
        print('Confusion matrix, without normalization')

    print(cm)
  
    fig, ax = plt.subplots()
    classes = vars.class_names#[unique_labels(y_true, y_pred)]
    n_classes = cm.shape[0]

    im = ax.imshow(ncm, interpolation='nearest', cmap=cmap) #try bilinear
    #ax.figure.colorbar(im, ax=ax)
    # We want to show all ticks...
    ax.set(xticks=np.arange(n_classes),
           yticks=np.arange(n_classes),
           # ... and label them with the respective list entries
           xticklabels=classes, yticklabels=classes)#,
           #title=title,
           #ylabel='True label', 
           #xlabel='Predicted label')
    ax.set_xticklabels(classes,fontsize = 13)
    ax.set_yticklabels(classes,fontsize = 13)
    ax.set_xlabel('Predicted label', fontsize = 13)
    ax.set_ylabel('True label', fontsize = 13)
    # Rotate the tick labels and set their alignment.
    # plt.setp(ax.get_xticklabels(), rotation=45, ha="right",
    #          rotation_mode="anchor")
    # plt.setp(ax.get_yticklabels(), rotation=45, ha="right",
    #          rotation_mode="anchor")
    ax.set_ylim((n_classes - 0.5, -0.5))
    #plt.setp(ax.get_xticklabels(), rotation='horizontal')

    # Loop over data dimensions and create text annotations.
    fmt = '.2f' if normalize else 'd'
    thresh = 30#cm.max() / 2.
    for i in range(cm.shape[0]):
        for j in range(cm.shape[1]):
            ax.text(j, i, format(cm[i, j], fmt), fontsize = 14, 
                    ha="center", va="center",
                    color="white" if cm[i, j] > thresh else "black")
    fig.tight_layout()
    plt.savefig('conf_mx.png')
    plt.pause(10)
    return ax